/*
 * Created on 2004-12-01
 *
 */
package edu.umk.mat.merdacz.JGFramework;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import edu.umk.mat.merdacz.JGServer.Player;

/**
 * Thread safe
 * 
 * @author Marcin Daczkowski
 *
 */
public class GameTable {
    int maxPlayers;
    HashMap table;
    Object lockTable = new Object();
    Object lockWinner = new Object();
    
    private Player winner = null;
    
    public GameTable(int maxPlayers) {
        this.maxPlayers = maxPlayers;
        
        table = new HashMap(maxPlayers);       
    }
    
    public boolean sitDown(Player p) {
        synchronized (lockTable) {
	        if ( (maxPlayers > table.size()) && (!table.containsValue(p)) ) {
	            table.put(p.getLoginName(), new GamePlayer(p));
	            return true;
	            
	        } else {
	            return false;
	        }
        }
    }
    
    public boolean standUp(Player p) {
        synchronized (lockTable) {
	        if (table.remove(p.getLoginName()) != null ) {	            
	            return true;
	        } else {
	            return false;
	        }
        }
        
    }
    
    public int emptyPlaces() {
        synchronized (lockTable) {
            return maxPlayers - table.size();
        }
    }

    public boolean isAllUnpaused() {
        synchronized (lockTable) {
            Iterator it = table.keySet().iterator();
            
            String gpln;
            GamePlayer gp;
            
            boolean allUnpaused = true;
            
            while (it.hasNext()) {
                gpln = (String) it.next();
                gp = (GamePlayer) table.get(gpln);
                
                if ( gp.isPause() ) {
                    allUnpaused = false;
                }
            }
            
            return allUnpaused;
        }
    }

    public boolean isAllReady() {
        synchronized (lockTable) {
            Iterator it = table.keySet().iterator();
            
            String gpln;
            GamePlayer gp;
            
            boolean allReady = true;
            
            while (it.hasNext()) {
                gpln = (String) it.next();
                gp = (GamePlayer) table.get(gpln);
                
                if ( !gp.isReady() ) {
                    allReady = false;
                }
            }
            
            return allReady;
        }
    }    
   
    public boolean isGameOver() {
        synchronized (lockTable) {
            synchronized (lockWinner) {
                Iterator it = table.keySet().iterator();
            
                String gpln;
                GamePlayer gp;
            
                boolean gameOver = true;
                int countNotLost = 0;
            
                while (it.hasNext()) {
                    gpln = (String) it.next();
                    gp = (GamePlayer) table.get(gpln);
                    
                    if ( !gp.isLost() ) {
                        winner = gp.getPlayer();
                        countNotLost++;
                    }
                
                    if (countNotLost > 1) {
                        gameOver = false;
                        winner = null;
                    }
                }
            
                return gameOver;
            }        
        }
    }
    
    public Player getWinner() {
        synchronized (lockWinner) {
            return winner;
        }
    }
    
    public void setDefeat(Player p) {
        synchronized (lockTable) {
            GamePlayer gp = (GamePlayer) table.get(p.getLoginName());
            gp.setLost(true);
        }
	}
    
    public boolean isDefeat(Player p) {
        synchronized (lockTable) {
            GamePlayer gp = (GamePlayer) table.get(p.getLoginName());
                        
            if (gp != null) {
                return gp.isLost();
            } else {
                return false;
            }
        }
    }
    
    public boolean switchPause(String playerName) {
        synchronized (lockTable) {
            if ( isByTable(playerName) ) {
                GamePlayer gp = (GamePlayer) table.get(playerName);
                gp.setPause(! gp.isPause());
                return true;
                
            } else {
                return false;
                
            }
        }
        
    }
  
    public boolean setReady(String playerName) {
        synchronized (lockTable) {
            if (isByTable(playerName)) {
                ((GamePlayer) table.get(playerName)).setReady(true);
                return true;
                
            } else {
                return false;
            }
        }
    }    
    
    /**
     * after we have winner we need to reset some values in out players HashMap
     * for example need to set all ready's to false etc
     */
    public void reset() {
        synchronized (lockTable) {
            Iterator it = table.keySet().iterator();
            
            String gpln;
            GamePlayer gp;
            
            while (it.hasNext()) {
                gpln = (String) it.next();
                gp = (GamePlayer) table.get(gpln);
                	
                gp.setLost(false);
                gp.setPause(false);
                gp.setReady(false);
            }            
        }
    }
    
    /**
     * Produces xml which contains all players sitting by the table
     * @return
     */
    public String toXml() {
        synchronized (lockTable) {
            String xmlout = "<table>";
            
            Set keys = table.keySet();
            Iterator it = keys.iterator();            
            
            while (it.hasNext()) {
                String pn = (String) it.next();
                GamePlayer gp = (GamePlayer) table.get(pn);
                xmlout = xmlout + gp.toXml();
                
            }
            
            xmlout = xmlout + "</table>";
            
            return xmlout;
        }
        
    }

    private boolean isByTable(String playerName) {
        synchronized (lockTable) {
            if (table.containsKey(playerName)) {
                return true;
            } else {
                return false;
            }
        }
        
    }
    
    
}
 