/*
 * Created on 2004-11-17
 *
 */
package edu.umk.mat.merdacz.JGServer;

import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * In memory Game representation. Takes care of restoring rooms from NXD.
 * Keeps {@link Rooms} for this game. Maintains {@link Players} in game 
 * (even those which has not yet joined any room). *  
 * 
 * @author Marcin Daczkowski
 *
 */

public class Game {

    /**
     * 
     */
	private Rooms rooms;
	
	/**
	 * 
	 */
	private Class gamelet;	
	
	/** 
	 * Players class is thread-save we don't have to
	 * synchornize!
	 */
	private Players players;
	
	/**
	 * Name of this room.
	 */
	private String name;
		
	/**	  
	 * @param name Name of game to create
	 * @param gamelet Game plugin
	 * @param roomsFromDb XML which contains all rooms which must be restored
	 */
	public Game(String name, Class gamelet, String roomsFromDb) {
		// create Rooms for this Game type
		rooms = new Rooms(gamelet);
		
		// restore state from NXD

        try {
    		SAXBuilder sb = new SAXBuilder();
    	    StringReader reader = new StringReader(roomsFromDb);
            
    	    sb.setValidation(false); 
    	    Document doc;
    	    
            doc = sb.build(reader);
            
            List rs = doc.getRootElement().getChildren("room");
            Iterator i = rs.iterator();
            
            while (i.hasNext()) {
                Element r = (Element) i.next();
               
                rooms.addRoom(new Room(
                        r.getAttributeValue("name"),
                        r.getAttributeValue("owner"),
                        r.getAttributeValue("type"),
                        r.getChild("invitations")));
                
    	        System.out.println("  " + "Room \"" + r.getAttributeValue("name") 
		                + "\" restored");
            }
            

            
        } catch (JDOMException e) {            
            System.out.println("dupa");
            // nothing we assume that exist produces correct xml
            
        } catch (IOException e) {
            // nothing we assume that exist produces correct xml
            
        }
		
		
		players = new Players();
		this.name = name;
	}
	
	/**
	 * I don't like that we just pass rooms outside. {@link Rooms} is synchronized
	 * but it is not clean solution (needed by ServerThread).
	 * 
	 * @return Reference to rooms in that game. 
	 */
	public Rooms getRooms() {
		return rooms;
		
	}

	/**
	 * @return Game name
	 */
	public String getName() {
	    return name;
	}
	
	/**
	 * @param p {@link Player} to add
	 */
	public void addPlayer(Player p) {
	    players.addPlayer(p);
	}
	
	/**
	 * @param p {@link Player} to remove
	 */
	public void delPlayer(Player p) {
        players.delPlayer(p);
	}
	
	/**
	 * @param name Player name to check for existence
	 * @return true if player already logged in false otherwise
	 */
	public boolean checkPlayer(String name) {
	    return players.checkPlayer(name);
	}
	
    /**
     * @return Plugin for this game
     */
    public Class getGamelet() {
        return gamelet;
    }
    
    /**
     * Sends given message to all players in game except playerToOmit.
     * It just wrappes same method from {@link Players}
     * 
     * @param xmlToSend message to broadcast
     * @param playerToOmit {@link Player} to omit
     * @throws IOException
     */
    public void broadcast(String xmlToSend, Player playerToOmit) throws IOException  {
        players.broadcast(xmlToSend, playerToOmit);

    }
       
	/**
	 * Just wraps same name method from Players (which syncs, and wrapps same
	 * from Player).
	 * 
	 * @param toPlayer
	 * @param xmlToSend
	 * @return
	 * @throws IOException
	 */
	public void sendMesageToPlayer(String toPlayer, String xmlToSend) throws IOException {
	    players.sendMessage(toPlayer, xmlToSend);
	    
	}
   
    /**
     * @return XML containing all actually logged in players names 
     */
    public String getPlayersInGame() {
        return players.getPlayersInXml();
    }
    


}
