/*
 * Created on 2004-11-16
 *
 */
package edu.umk.mat.merdacz.JGServer;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import org.jdom.Document;
import org.jdom.Element;

/**
 * In memory game room representation.
 * <p>
 * It should be probably reinvented in future, because at the moment
 * it doesn't take care of NXD updates (which is actually done external in
 * {@link ServerThread}. Tho It will need rewritting {@link Game} also :( 
 * 
 * @author Marcin Daczkowski
 *
 */

public class Room {

    /**
     * 
     */
	private Players players;
	
	/**
	 * 
	 */
	private String name;
	
	/**
	 * 
	 */
	private String owner;
	
	/**
	 * 
	 */
	private String type;
	
	/**
	 * 
	 */
	private Object lockPlayers = new Object();
	
	/**
	 * 
	 */
	private Object lockPlugin = new Object();
	
	/**
	 * 
	 */
	private Object lockInvitations = new Object();
	
	/** 
	 * give, take away and modify invitations possible
	 * only for owner of channel 
	 */	
	private TreeSet invitations;

	/** 
	 * used for keeping some per Room data like cache for boxes in 
	 * Netris. And for parsing specific per game requests. 
	 */
	private Gamelet plugin;
	
	/** 
	 * used to instantinate by ServerThread in its main while-loop
	 * 
	 * TODO is it good to do sth like Class s = r.plugin.getClass() instead?
	 * TODO getter + synchronized? (only reading) -> return new Class ga..
	 */
	private Class gamelet;
	
	/**
	 * Used for two main purposes:
	 * 	1) to auto leave players after it has been removed (they have to 
	 * 		free reference in order to GC take care of it)
	 *  2) TODO to disallow new joins (is it important when we have 1) ?)
	 * 
	 */
	private boolean roomState;
	
	/**
	 * there are two essentialy diffrent scenarios of creating room
	 * first one is when the player wants to set up his room
	 * second when server is restarting. in second case we 
	 * have to read data from NXD 
	 * 
	 * @param name
	 * @param owner
	 * @param type
	 */
	public Room(String name, String owner, String type, Element invitations) {
		players = new Players();
		this.invitations = new TreeSet();
	
		this.name = name;
		this.owner = owner;
		this.type = type;

		roomState = false;
		
		// put all invitations into TreeSet
		if ( invitations != null ) { // null-test just to be sure
		    List l = invitations.getChildren("invite");
		    Iterator it = l.iterator();
		    
		    Element inv;
		    while (it.hasNext()) {
		        inv = (Element) it.next();
		        
		        synchronized (lockInvitations) {
		            this.invitations.add(inv.getAttributeValue("playerName"));
		        }
		        
		    }
		}

	}
	

	/**
	 * @param p
	 */
	public boolean join(Player p) {
	    // lock invitations
	    synchronized (lockPlayers) {
	        synchronized (lockInvitations) {
	            if (p.getLoginName().equals(owner) || type.equals("public") 
	                    || invitations.contains(p.getLoginName())) { 
	                players.addPlayer(p);
	                return true;
	            } else {
	                return false;
	            }
	        }
	    }
	}
		
	/**
	 * @param p
	 */
	public void leave(Player p) {
	    synchronized (lockPlayers) {
	        players.delPlayer(p);
	    }
	}
	
	public void grantInvite(String playerName) {
	    synchronized (lockInvitations) {
	        invitations.add(playerName);
	    }
	}
	
	/**
	 * @param gamelet
	 */
	public void setGamelet(Class gamelet) {		 
	    
		this.gamelet = gamelet;
		
		
		try {
			Constructor gameletConstructor = gamelet.getConstructor(null);
			plugin = (Gamelet) gameletConstructor.newInstance(null);
			
		} catch (NoSuchMethodException e) {
			System.out.println(e);
			
		} catch (InstantiationException e) {
			System.out.println(e);
			
		} catch (IllegalAccessException e) {
			System.out.println(e);
			
		} catch (InvocationTargetException e) {
			System.out.println(e);
			
		}
	}
	
	/**
	 * we have to destroy room when it is removed from array (Rooms
	 * class). destorying implies: removing internal objects (like plugin).
	 * and informing all Players that the room they are in has been removed.
	 * destroying room doesn't check credentials (should it?) beacuse
	 * they were checked when removing from array (should be -> not yet)
	 */	
	public void delete() {
	
	}		
	
    /**
     * @return
     */
    public String getName() {
        return name;
    }
    
    
    /**
     * @return
     */
    public String getOwner() {
        return owner;
    }
    
    
    /**
     * @return
     */
    public boolean isRemoved() {
        return roomState;
    }
    
    /**
     *
     */
    public void removeMe() {
        roomState = true;
    }
    
    /**
     * Because in most games some data must be shared, or be persistent between
     * requests we need to be aware that some synchronization must be made. 
     * Unless we don't want plugins developers to have a lot of mess with 
     * synchronizing this per room synchronized parseRequest has been written.
     * 
     * @param id
     * @param player
     * @param doc
     * @return 
     */
    public String parseRequest(Player player, Document doc) {
        String xmlout;
        synchronized (lockPlugin) {
            xmlout = plugin.parseRequest(player, doc);
        }
        return xmlout;
    }
    
    /**
     * @param xmlToSend
     * @param playerToOmit
     * @throws IOException
     */
    public void broadcast(String xmlToSend, Player playerToOmit) throws IOException {
        synchronized (lockPlayers) {
            players.broadcast(xmlToSend, playerToOmit);           
        }
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {                
        synchronized(lockPlayers) {
            return "<room players=\""+ players.size()  +"\">"
            	+ "<name>" + name + "</name>"
            	+ "<type>" + type + "</type>"
            	+ "<owner>" + owner + "</owner>"
            	+ "</room>";
        }    
    }
    
    /**
     * yeah ! yeah! yeah!
     * @return
     */
    public String getPlayersInRoom() {
        return players.getPlayersInXml();
    }
}
