/*
 * Created on 2004-11-16
 *
 */
package edu.umk.mat.merdacz.JGServer;


import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import edu.umk.mat.merdacz.JGServer.config.CfgBox;
import edu.umk.mat.merdacz.JGServer.config.CfgLoader;
import edu.umk.mat.merdacz.JGServer.config.CfgLoaderException;
import edu.umk.mat.merdacz.JGServer.config.ServerCfg;

/** 
 * Main server loop. We listen for connection and
 * create separate thread for each one
 * 
 * @author Marcin Daczkowski
 *
 */
public class Server {
	/**
	 * 
	 */
    private ServerSocket ss;
	
	/**
	 * 
	 */
	private ServerCfg cfg;
	
	/**
	 * 
	 */
	private Games games;
	
	/**
	 * Need for restoring rooms from db after server reboot
	 */
	private DbWrapper db;
	 	
	/**
	 * @param configFileName
	 */
	public Server(String configFileName) {	     
		try {
		    // Bind object cfg with configuration xml file
		    CfgBox cfgBox = new CfgBox(cfg);
		    
            CfgLoader cfgLoader = new CfgLoader( cfgBox, configFileName );
            cfg = cfgBox.getCfg();

    		// Show some loaded configuration details if in debug mode
    		if ( cfg.getDebug() ) {
    		    System.out.println(cfg);		    
    		}
    		
    		// Create instance of dbWrapper class - we pass
    		// address of db to it (from cfg file)
    		
    		db = new DbWrapper(cfg.getDbAddress(), cfg.getDbUser(), 
    		        cfg.getDbPass());
    			
    		// load games...
    		games = new Games(cfg, db.dbGetRooms());
   		
    		//System.out.println(db.testMe());
    		
    		// Open port and wait for incoming connections
  			listen(cfg.getPort());  		
    		
        } catch (CfgLoaderException e) {
            System.out.println(e.getMessage());
        } catch (IOException  e) {
			System.out.println(e.getMessage());
		} catch (DbWrapperException e) {
		    System.out.println(e.getMessage());
        }		
        
	}

	/**
	 * @param port
	 * @throws IOException
	 */
	private void listen(int port) throws IOException {		
		ss = new ServerSocket(port);		
		
		if ( cfg.getDebug() ) {
		    System.out.println("Listening on " + ss);
		}
		
		// main while-loop of server. every new incoming connection forks new thread
		while (true) {
			Socket s = ss.accept();
		
		    if (cfg.getDebug()) {
		        System.out.println("Connection from " + s);
		    }
	
			new ServerThread(this, s);
		}				 
	}
	
	/**
	 * @param s
	 */
	void removeConnection(Socket s) {
	    if (cfg.getDebug()) {
	        System.out.println("Removed connection from " + s);
	    }
		    

		try {
			s.close();
		} catch (IOException e) {
			System.out.println("Error closing " + s);
			e.printStackTrace();
		}
	}

	/**
	 * @return games that have been deployed
	 */
	public Games getGames() {
	    return games;
	}
		
    /**
     * @return Returns the cfg.
     */
    public ServerCfg getCfg() {
        return cfg;
    }
    
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// default place to keep configuration 
		String configFile = "config/server.xml";
		
		// parse command line for "-config" option		 		
		if ( (args.length == 2) && (args[0].equals("-config") ) ) {
			File f = new File(args[1]);
			if ( f.exists() ) {
				configFile = args[1];				
			} else {
				System.err.println("Error: Can't find \"" + args[1] + "\".");
				System.err.println();
				System.exit(-1);
			}
		} else if ( args.length  != 0 ) { 
		    //print help
		    System.err.println("Usage: java -jar JGServer.jar [-config file.xml]" );
		    System.err.println();
		    System.exit(-1);
		}
									
		// just launch server
		try{	
			new Server(configFile);
		} catch (NumberFormatException e) {
			System.err.println(e.getMessage());		
		}		
		
		
	}
}
