/*
 * Created on 2004-11-19
 *
 */
package edu.umk.mat.merdacz.JGServer.config;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.JDOMParseException;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;

/**
 * CfgLoader is used to fill in {@link ServerCfg} object. 
 * We take data from xml file parse it by SAXParser (Xerces 
 * underlies) and use simple JDOM API to get interesting 
 * content. We put it CfgBox - the boxing is not really needed,
 * it was used because of apache-commons digester was used 
 * to bind config before.
 * <p>
 * And yes we don't really like digester!!! 
 *
 * @author Marcin Daczkowski
 *
 */
public class CfgLoader {

	/**
	 * Fills in {@link ServerCfg} object embeded in {@link CfgBox}
	 * opaque class. 
	 * @throws CfgLoaderException
	 * @throws ParserConfigurationException
	 */
	public CfgLoader(CfgBox cfgBox, String configFile) throws CfgLoaderException {   
	        
        SAXBuilder sb = new SAXBuilder("org.apache.xerces.parsers.SAXParser", true);          
        sb.setFeature("http://apache.org/xml/features/validation/schema",true);
        sb.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation","server.xsd");
        
        Document doc;      
                
        try {
            doc = sb.build("config/server.xml");    

            Element root = doc.getRootElement();
            // General options for server 
            ServerCfg conf = new ServerCfg();
            conf.setVersion(root.getAttributeValue("version"));            
            conf.setPort(root.getAttribute("port").getIntValue()); 
            conf.setDebug(root.getAttribute("debug").getBooleanValue());
            
            // Get information about plugins
            XPath xp = XPath.newInstance("//plugin");              
            List l = xp.selectNodes(doc);
            Iterator i = l.iterator();
            
            while (i.hasNext()) {                
                Element e = (Element) i.next();                             
                
                PluginCfg plugin = new PluginCfg();
                plugin.setName(e.getAttributeValue("name"));
                plugin.setVersion(e.getAttributeValue("version"));
                plugin.setClazz(e.getAttributeValue("class"));
                
                conf.addPlugin(plugin);                                
            }
            
            // Get keys information
            Element keys = root.getChild("keys");
            KeyCfg publ = new KeyCfg();
            publ.setKeyFile(keys.getChild("public").getAttributeValue("file"));
            conf.setPublicKey(publ);
            
            KeyCfg priv = new KeyCfg();
            priv.setKeyFile(keys.getChild("private").getAttributeValue("file"));
            priv.setKeyPassword(keys.getChild("private").getAttributeValue("password"));
            conf.setPrivateKey(priv);
            
            // Get db information
            // Should be probably wrapped in sth like DbCfg !
            Element aDb = root.getChild("db");            
            conf.setDbUser(aDb.getChild("user").getText());
            conf.setDbPass(aDb.getChild("password").getText());
            conf.setDbAddress(aDb.getChild("address").getText());
            
            
            
            // We don't really need that boxing class when not using this *** digester
            cfgBox.setCfg(conf);            
            
        } catch (JDOMParseException e) {
            throw new CfgLoaderException(e.getMessage(), e);
            
        } catch (JDOMException e) {
            throw new CfgLoaderException(e.getMessage(), e);
            
        } catch (IOException e) {
            throw new CfgLoaderException(e.getMessage(), e);
            
        }
	    
	    
	}
	

}
