/*
 * Created on 2004-11-19
 *
 */
package edu.umk.mat.merdacz.JGServer.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 * Main class used to keep server configuration.
 * It is filled with data from xml configuration file.
 * 
 * After filling this cfg by CfgLoader we are only reading
 * hence no synchronization here  
 * 
 * @author Marcin Daczkowski
 *
 */
public class ServerCfg {
    
    private HashMap plugins;
    
    private KeyCfg privateKey;
    private KeyCfg publicKey;
    
    private String version;
    private int port;
    private boolean debug;
    
    private String dbAddress;
    private String dbUser;
    private String dbPass;
    
	public ServerCfg() {
	    plugins = new HashMap();
	    // really need it ?
	    version = new String();	    
	}
	
	
    /**
     * @return Debug mode
     */
    public boolean getDebug() {
        return debug;
    }
    /**
     * @param debug Debug mode
     */
    public void setDebug(boolean debug) {
        this.debug = debug;
    }
    /**
     * @return The port on which the server should listen
     */
    public int getPort() {
        return port;
    }
    /**
     * @param port The port on which the server should listen
     */
    public void setPort(int port) {
        this.port = port;
    }
    /**
     * @return The version of server
     */
    public String getVersion() {
        return version;
    }
    /**
     * @param version The version of server
     */
    public void setVersion(String version) {
        this.version = version;
    }
    
    /**
     * Usefull if client asks server for all supported games.
     * 
     * @return The list of keys from internal plugins HashTable 
     */
    public ArrayList getPluginNamesList() {
        ArrayList l = new ArrayList();
        Set keys = plugins.keySet();
        Iterator keysi = keys.iterator();
        
        while (keysi.hasNext()) {
            l.add(keysi.next());
        }
        
        return l;
    }

    public void addPlugin(PluginCfg plugin) {
	    plugins.put(plugin.getName(), plugin);
	}
	
	public PluginCfg getPlugin(String pluginName) {
	    return (PluginCfg) plugins.get(pluginName);
	}
	/**
     * @return Server private key for ssl connection 
     */
    public KeyCfg getPrivateKey() {
        return privateKey;
    }
    
    /**
     * @param privateKey Server private key for ssl connection
     */
    public void setPrivateKey(KeyCfg privateKey) {
        this.privateKey = privateKey;
    }
    /**
     * @return Server public key for ssl connection
     */
    public KeyCfg getPublicKey() {
        return publicKey;
    }
    /**
     * @param publicKey Server public key for ssl connection
     */
    public void setPublicKey(KeyCfg publicKey) {
        this.publicKey = publicKey;
    }
  
    /**
     * @return
     */
    public String getDbAddress() {
        return dbAddress;
    }
    
    /**
     * @param dbAddress
     */
    public void setDbAddress(String dbAddress) {
        this.dbAddress = dbAddress;
    }
    
    /**
     * @return
     */
    public String getDbPass() {
        return dbPass;
    }
    
    /**
     * @param dbPass
     */
    public void setDbPass(String dbPass) {
        this.dbPass = dbPass;
    }
    
    /**
     * @return
     */
    public String getDbUser() {
        return dbUser;
    }
    
    /**
     * @param dbUser
     */
    public void setDbUser(String dbUser) {
        this.dbUser = dbUser;
    }
    
    /**
     * Produces report about configuration with witch server
     * is actually running.   
     *   
     */
    public String toString() {
        String nl = System.getProperty("line.separator");
        StringBuffer buf = new StringBuffer();
        
        
        // general server options
        buf.append("Server:").append(nl);
	    buf.append("  version: " + getVersion() +
	            " port: " + getPort() + " debug: " +
	            getDebug()).append(nl);

	    // iterate through all plugins and append some info
	    // about each of them
	    buf.append("Plugins:").append(nl);
	    
	    ArrayList list = getPluginNamesList();
	    Iterator listi = list.iterator();
	    
	    while (listi.hasNext()) {
	        PluginCfg p = getPlugin((String) listi.next());
		    buf.append("  " + p.getName() + " -> ");
		    buf.append("version: " + p.getVersion() + " class: "
		            + p.getClazz()).append(nl);
	        
	    }
	    
	    // keys for SSL 
	    buf.append("Keys: ").append(nl);
	    buf.append("  " + getPublicKey().getKeyFile()).append(nl);
	    buf.append("  " + getPrivateKey().getKeyFile() + " "
	            + this.getPrivateKey().getKeyPassword()).append(nl);
	    
	    // db
	    
	    buf.append("Db:").append(nl);
	    buf.append("  " + "Address: " + getDbAddress()).append(nl);
	    buf.append("  " + "User: " + getDbUser() + "  Password: " + getDbPass());
	    buf.append(nl);
	    
	    return buf.toString();
    }
}
