/*
 * Created on 2004-12-01
 *
 */
package edu.umk.mat.merdacz.JGServer.plugins.Netris;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;

import edu.umk.mat.merdacz.utils.EmptyQueueException;
import edu.umk.mat.merdacz.utils.Queue;

/**
 * @author Marcin Daczkowski
 *
 */
public class BlocksManager {
   
    private boolean depsStarted;
    private HashMap blocks;
    private Object lockBlocks = new Object();
    
    private int maxBlockType = 10; 
    
    public BlocksManager() {
        depsStarted = false;
        blocks = new HashMap();
    }
    
    /**
     * can't be triggered after we start producing blocks
     *  
     * @param playerName
     * @throws BlocksManagerException
     */
    public void registerNewPlayer(String playerName) throws BlocksManagerException {
        if ( !depsStarted ) {
            synchronized (lockBlocks) {
                blocks.put(playerName, new Queue());
                
            }
        }
        else {
            throw new BlocksManagerException();
        }
                
    }
    
    
    /**
     * We need to do this after standUp 
     * 
     * @param playerName
     * @throws BlocksManagerException
     */
    public void unregisterPlayer(String playerName) throws BlocksManagerException {
        if ( !depsStarted ) {
            synchronized (lockBlocks) {
                // we do not check if he was registered before
                blocks.remove(playerName);
            }
        } else {
            throw new BlocksManagerException();
        }
    }
    
    public void start() {
        depsStarted = true;
        
    }
    
    /**
     * 
     * @param playerName 
     * @param count
     * @return
     * @throws BlocksManagerException
     */
    public String getBlocks(String playerName, int count) throws BlocksManagerException {
        if ( depsStarted ) {
            synchronized (lockBlocks) {
                // all must be synchronized beacause we don't want 
                // two players to call produceBlocks simult.
                
                Queue q = (Queue) blocks.get(playerName);
                
                if ( q == null ) {
                    throw new BlocksManagerException();
                    
                }
                
                String xmlout = "<blocks>";
                
                for (int i = count; i > 0; ) {
                    try {
                        xmlout = xmlout + "<block type=\"" +
                        	(Integer) q.dequeue() + "\" />";  
                        i--;
                        
                    } catch (EmptyQueueException e) {
                        produceBlocks(count);
                        
                    }
                }
                
                xmlout = xmlout + "</blocks>";
                
                return xmlout;
            }
            
        } else {
            throw new BlocksManagerException();
            
        }
    }
    
    /**
     * @param count how much new block to produce
     */
    private void produceBlocks(int count) {
        if (depsStarted) {
            synchronized (lockBlocks) {                
                // produce random blocks array
                Random rand = new Random();
                ArrayList randBlocks = new ArrayList();
                
                for (int i = count - 1; i >= 0; i--) {
                    randBlocks.add(new Integer(rand.nextInt(maxBlockType-1)));
                }
                
                // append to all registered players queues                
                Set keys = blocks.keySet();
                Iterator it = keys.iterator();
                
                while (it.hasNext()) {
                    String pn = (String) it.next();
                    Queue q = (Queue) blocks.get(pn);
                    
                    for (int i = count - 1; i >= 0; i--) {
                        q.enqueue(randBlocks.get(i));
                    }
                }
            }
        }
        
    }
    
    /** 
     * after we have winner we need to reset all queues
     * and back to state that we can register/unregister players 
     *
     */
    public void reset() {
        depsStarted = false;
        blocks = new HashMap();
        
    }

}
