/*
 * Created on 2004-11-17
 *
 */
package edu.umk.mat.merdacz.JGServer.plugins.Netris;


import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import edu.umk.mat.merdacz.JGFramework.GameReplies;
import edu.umk.mat.merdacz.JGFramework.GameState;
import edu.umk.mat.merdacz.JGFramework.GameTable;
import edu.umk.mat.merdacz.JGServer.Gamelet;
import edu.umk.mat.merdacz.JGServer.Player;

/**
 * 
 * It is externaly "locked" by lockPlugin in {@link Room} 
 * to prevent such scenario:
 * p1, p2, p3 sitsDown; p1, p2, p3 startGame
 * while p3 startGame reaches isAllReady it gives true
 * and thread is switched to p2 which i.e standsUp or
 * p4 sitsDown
 * <p>
 * One parseRequest is runned at time. This is good beacuse
 * we do not have to synchronize internally. But for efficience
 * and scalabilty is should be done internally (via many locks
 * on different resources) 
 * <p>
 * All parts of JGFramework are written properly (they synchronizes)
 * this only this plugin "problem".
 * 
 * 
 * @author Marcin Daczkowski
 *
 */
public class Netris implements Gamelet {

    GameTable table;
    GameState state;
    
    BlocksManager blocksManager;
    
    boolean depsByTable;
    
    // shouldn't be hardcoded!
    public static int blocksToGrab = 10;
    
	/**
	 * TODO give plugin *somehow* access those parts of configuration
	 * which belongs to it
	 */
	public Netris() {
	    // TODO musn't be hardcoded
	    int maxPlayers = 2;
	    // TODO minPlayers !!!! startGame should check it.
	    // moreover both min/maxPlayers should be placed in game configuration
	    // or should be able to set it by owner of room (or both)
	    
	    table = new GameTable(maxPlayers);
	    state = new GameState();
	    blocksManager = new BlocksManager();
	    
	    depsByTable = false;
  	}


	/**
	 * 
	 */
	public String parseRequest(Player player, Document doc) {


	    String xmlout;

	    Element root = doc.getRootElement();	    
	    String type = root.getAttributeValue("type");
	    
	    // TODO need to do some checking 
	    
	    // if player has defeated we don't parse any req until 
	    // game over
	    
	    if (table.isDefeat(player)) {
	        return GameReplies.replyFailed(type, "you have defeated");
	    }
	    	    
	    if (type.equals("sitDown")) { 
	        xmlout = sitDown(player); 	
	        
	    } else if (type.equals("standUp")) { 
	        xmlout = standUp(player);
	        
	    } else if (type.equals("getTable")) { 
	        xmlout = getTable();
	        
	    } else if (type.equals("startGame")) { 
	        xmlout = startGame(player);
	        
	    } else if (type.equals("switchPause")) { 
	        xmlout = switchPause(player);	  

	    } else if (type.equals("getBlocks")) { 
	        xmlout = getBlocks(player);
	    
	    } else if (type.equals("moveBlock")) { 
	        xmlout = moveBlock(player, root.getChild("move"));
	        
	    } else if (type.equals("defeat")) {
	        xmlout = defeat(player);
	       
	    } else {	     
	        return GameReplies.replyFailed(type, "not implemented");
	        
	    }		
	    
	    return xmlout;
		
	}


    /**
     * @param player
     * @return
     */
    public String sitDown(Player player) {
        if ( state.getState().equals(GameState.STOPPED)) { 
            if ( table.sitDown(player) ) {
                try {
                    blocksManager.registerNewPlayer(player.getLoginName());
                    depsByTable = true;
                    
                } catch (BlocksManagerException e) {
                    // shouldn't happen ;-)
                }
                
                return 
                	GameReplies.replyOK("sitDown") + "\n" +
                	GameReplies.gameBroadcast("sitDown",
                	        player.toXml());
                	        
                
            } else {
                // should be able to distinguish between reasons
                return 
                	GameReplies.replyFailed("sitDown", "no places available" +
                			"or you already sitting by table");
                
            }
        } else {
            return 
            	GameReplies.replyFailed("sitDown", "game has been started");
            
        }
    }

    /**
     * Consider using String playerLoginName not Player 
     * @param id
     * @param player
     * @return
     */
    public String standUp(Player player) {
        if ( state.getState().equals(GameState.STOPPED)) {
            if ( table.standUp(player) ) {
                try {
                    blocksManager.unregisterPlayer(player.getLoginName());
                    depsByTable = false;
                } catch (BlocksManagerException e) {
                    // shouldn't happen ;-)
                    
                }
                return 
                	GameReplies.replyOK("standUp") + "\n" +
                	GameReplies.gameBroadcast("standUp",
                	        player.toXml());
                	
                
            } else {
                return 
                	GameReplies.replyFailed("standUp", "you are not sitting by table");
            }
        } else {
            return 
            	GameReplies.replyFailed("standUp", "game has been started");
        }
        
    }

    
    /**
     * Gets players by table and some additional info like how much empty places
     * we still have
     * 
     * @param id
     * @return
     */
    public String getTable() {
        return 
        	GameReplies.reply("getTable", table.toXml());
        
    }
    
    /**
     * 
     * @param id
     * @param player
     * @return
     */
    public String startGame(Player player) {
                
        if (state.getState().equals(GameState.STOPPED))  {
            if (! table.setReady(player.getLoginName()) ) {
                return 
                	GameReplies.replyFailed("startGame", "you are not sitting by table"); 
            }
            
            // TODO check if user has already started (not necessary
            // just for consistence)
            
            String xmlout = GameReplies.replyOK("startGame") + "\n" ;
            
            // some info (used by client to manage displaying some 
            // proper info on screen)
            
            xmlout = GameReplies.gameBroadcast("startGame", 
                    player.toXml()) + "\n";
            
            if (table.isAllReady()) {
                state.setStarted();
                blocksManager.start();
                
                xmlout = xmlout + 
                	GameReplies.gameBroadcast("startGame", "<started/>");
                
            }
            
            return xmlout;
            
        } else {
            return 
            	GameReplies.replyFailed("startGame", "Game has been already started");
        }
        
    }

    /**
     * 
     * @param id
     * @param player
     * @return
     */
    public String switchPause(Player player) {
        
        if ( !state.getState().equals(GameState.STOPPED)) {
            if ( ! table.switchPause(player.getLoginName())) {
                return 
                	GameReplies.replyFailed("switchPause", 
                	        "you are not sitting by table"); 
            }
            
            String xmlout = GameReplies.replyOK("switchPause") + "\n";
            
            // we pass it because it is easier for client to manage displaying 
            // proper information
            xmlout = xmlout + GameReplies.gameBroadcast("switchPause", player.toXml()) + "\n";            
            
            if (table.isAllUnpaused()) {
                state.setStarted();
               
                xmlout = xmlout + GameReplies.gameBroadcast("switchPause", 
                        "<gameState>unpaused</gameState>") + "\n";
                
            } else if (state.getState().equals(GameState.STARTED)) {
                    state.setPaused();
                    
                    xmlout = xmlout + GameReplies.gameBroadcast("switchPause", 
                	"<gameState>paused</gameState>");                    
            }
            
            return xmlout;
            
            
        } else {
            return 
    			GameReplies.replyFailed("pauseGame", "Game hasn't been started yet");
        }
            
    }

    /**
     * 
     * @param id
     * @param player
     * @return
     */
    public String getBlocks(Player player) {
        if (state.getState().equals(GameState.STARTED)) {
            try {
                return GameReplies.reply("getBlocks", 
                	blocksManager.getBlocks(player.getLoginName(), blocksToGrab) );
                	
                
            } catch (BlocksManagerException e) {
                // player not by table or BM not started, cause we check game state above 
                // this must be the first one
                
                // TODO shouldn't happen ... ?
                return 
                	GameReplies.replyFailed("getBlocks", "you are not sitting by table");
            }
            
        } else if ( state.getState().equals(GameState.STOPPED)) {            
            return 
            	GameReplies.replyFailed("blockMove", "game not started");
            	
        } else { // game paused
            return 
            	GameReplies.replyFailed("blockMove", "game paused");
        }
        
           
    }
    
    /**
     * 
     * @param id
     * @param player
     * @return
     */
    public String moveBlock(Player player, Element move) {
        if (move == null) {
            return GameReplies.replyFailed("parseError", "move element missing");
        }

		// TODO check what's in move element (talk to michal to state this)    	
        
        if (state.getState().equals(GameState.STARTED)) {
            move.setAttribute("from", player.getLoginName());
            XMLOutputter xo = new XMLOutputter();
            
            String xmlout;
            
            xmlout = GameReplies.gameBroadcast("blockMove",
                    xo.outputString(move)) + "\n" +
                    GameReplies.replyOK("blockMove");
            
            return 
            	xmlout;
            
        } else if ( state.getState().equals(GameState.STOPPED)) {            
            return 
            	GameReplies.replyFailed("blockMove", "game not started");
            	
        } else { // game paused
            return 
            	GameReplies.replyFailed("blockMove", "game paused");
        }
        
    }

    /**
     * Used to both announce that we lost or that we are giving up
     * 
     * @param id
     * @param player
     * @return
     */
    public String defeat(Player player) {
       
        if (state.getState().equals(GameState.STARTED)) {
            
            table.setDefeat(player);
            
            String xmlout = GameReplies.replyOK("defeat") + "\n";
            
            xmlout = xmlout + GameReplies.gameBroadcast("defeat", player.toXml()) + "\n";
            
            if (table.isGameOver()) {
                Player winner = table.getWinner();
                
                xmlout = xmlout + GameReplies.gameBroadcast("gameOver", 
                        "<winner>" + winner.toXml() + "</winner>");
                
                // TODO some stats
                
                table.reset();
                blocksManager.reset();
                
                state.setStopped();
                
            }
            
            return xmlout;
            
        } else {
            return 
            	GameReplies.replyFailed("pauseGame", "Game hasn't been started yet");
        }        
        
    }    
}
